package org.gcube.datacatalogue.grsf_manage_widget.client.view.subwidgets;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.gcube.datacatalogue.common.enums.Product_Type;
import org.gcube.datacatalogue.grsf_manage_widget.client.GRSFManageWidgetServiceAsync;
import org.gcube.datacatalogue.grsf_manage_widget.shared.ConnectedBean;
import org.gcube.datacatalogue.grsf_manage_widget.shared.ManageProductBean;

import com.github.gwtbootstrap.client.ui.Button;
import com.github.gwtbootstrap.client.ui.CheckBox;
import com.github.gwtbootstrap.client.ui.Icon;
import com.github.gwtbootstrap.client.ui.Paragraph;
import com.github.gwtbootstrap.client.ui.TextBox;
import com.github.gwtbootstrap.client.ui.constants.ButtonType;
import com.github.gwtbootstrap.client.ui.constants.IconType;
import com.google.gwt.core.client.GWT;
import com.google.gwt.core.client.Scheduler;
import com.google.gwt.core.client.Scheduler.ScheduledCommand;
import com.google.gwt.dom.client.Style.Float;
import com.google.gwt.dom.client.Style.FontWeight;
import com.google.gwt.dom.client.Style.Unit;
import com.google.gwt.event.dom.client.ChangeEvent;
import com.google.gwt.event.dom.client.ChangeHandler;
import com.google.gwt.event.dom.client.ClickEvent;
import com.google.gwt.event.dom.client.ClickHandler;
import com.google.gwt.event.dom.client.KeyPressEvent;
import com.google.gwt.event.dom.client.KeyPressHandler;
import com.google.gwt.uibinder.client.UiBinder;
import com.google.gwt.uibinder.client.UiField;
import com.google.gwt.user.client.rpc.AsyncCallback;
import com.google.gwt.user.client.ui.Anchor;
import com.google.gwt.user.client.ui.Composite;
import com.google.gwt.user.client.ui.HTML;
import com.google.gwt.user.client.ui.HorizontalPanel;
import com.google.gwt.user.client.ui.VerticalPanel;
import com.google.gwt.user.client.ui.Widget;

/**
 * Connections panel
 * @author Costantino Perciante at ISTI-CNR (costantino.perciante@isti.cnr.it)
 */
public class ConnectToWidget extends Composite{

	private static ConnectToWidgetUiBinder uiBinder = GWT
			.create(ConnectToWidgetUiBinder.class);

	private static final String REGEX_UUID = "[0-9a-f]{8}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{4}-[0-9a-f]{12}";

	interface ConnectToWidgetUiBinder extends UiBinder<Widget, ConnectToWidget> {
	}

	@UiField
	VerticalPanel connectPanel;

	@UiField
	Button suggestRecord;

	private List<Tuple> connectList = new ArrayList<Tuple>(0); // for the suggested ones
	private List<ConnectedBean> currentlyConnected; // they can be "unconnected" or "removed"
	private List<ConnectedBean> suggestedByKnowledgeBase; // they can be "connected"
	private GRSFManageWidgetServiceAsync service;

	public ConnectToWidget(final ManageProductBean bean, GRSFManageWidgetServiceAsync service) {
		initWidget(uiBinder.createAndBindUi(this));

		this.service = service;

		// get already connected beans, and suggested ones
		currentlyConnected = bean.getCurrentConnections();

		if(currentlyConnected != null){
			connectPanel.add(new HTML("<hr style=\"width:100%;\"/>"));
			for (ConnectedBean connected : currentlyConnected) {
				Widget widget = buildWidgetForConnected(connected, false);
				connectPanel.add(widget);
				connectPanel.add(new HTML("<hr style=\"width:100%;\"/>"));
			}
		}

		// add the one suggested by the knowledge base, they can be just "connected"
		suggestedByKnowledgeBase = bean.getSuggestedByKnowledgeBaseConnections();

		if(suggestedByKnowledgeBase != null){
			for (ConnectedBean connected : suggestedByKnowledgeBase) {
				Widget widget = buildWidgetForConnected(connected, true);
				connectPanel.add(widget);
				connectPanel.add(new HTML("<hr style=\"width:100%;\"/>"));
			}
		}

		// manage the button for manual suggestion
		final String acceptedDomain = bean.getGrsfDomain().equalsIgnoreCase(Product_Type.STOCK.getOrigName()) ? 
				Product_Type.FISHERY.getOrigName()  : Product_Type.STOCK.getOrigName(); 
				suggestRecord.setTitle("Connect this " + bean.getGrsfDomain() + " record to a " + acceptedDomain + " record ");
				suggestRecord.setText("Add Connection");
				suggestRecord.setType(ButtonType.LINK);
				suggestRecord.getElement().getStyle().setFontWeight(FontWeight.BOLD);
				suggestRecord.getElement().getStyle().setFloat(Float.RIGHT);

				// add handler
				suggestRecord.addClickHandler(new ClickHandler() {

					@Override
					public void onClick(ClickEvent arg0) {

						ConnectedBean cb = new ConnectedBean();
						Tuple t = buildWidgetConnect(cb, acceptedDomain);
						connectList.add(t);
						connectPanel.add(t.getW());

					}
				});

	}

	/**
	 * 
	 * @param similarGRSFRecord
	 * @return
	 */
	public static Widget buildWidgetForConnected(final ConnectedBean connected, boolean suggestedByKb){
		HorizontalPanel hp = new HorizontalPanel();
		hp.setWidth("95%");
		VerticalPanel leftPanel = new VerticalPanel();
		leftPanel.setWidth("80%");
		leftPanel.getElement().getStyle().setMarginLeft(20, Unit.PX);
		Paragraph name = new Paragraph("Record Name: " + (connected.getTitle() != null? connected.getTitle() : "Unavailable"));
		leftPanel.add(name);

		Paragraph semanticIdentifier = new Paragraph("Semantic Identifier: " + 
				connected.getSemanticIdentifier());
		leftPanel.add(semanticIdentifier);

		Paragraph identifier = new Paragraph("Record UUID: " + 
				connected.getKnowledgeBaseId());
		leftPanel.add(identifier);

		Anchor view = new Anchor();
		view.setHref(connected.getUrl());
		view.setText("View"); 
		view.setTitle("Click to view the similar record");
		view.setTarget("_blank");
		view.getElement().getStyle().setFontWeight(FontWeight.BOLD);
		leftPanel.add(view);

		VerticalPanel rightPanel = new VerticalPanel();
		rightPanel.setWidth("20%");
		rightPanel.getElement().getStyle().setFloat(Float.RIGHT);

		if(!suggestedByKb){
			final CheckBox removeExtra = new CheckBox("Remove");
			removeExtra.setTitle("Remove this connection");
			removeExtra.addClickHandler(new ClickHandler() {

				@Override
				public void onClick(ClickEvent arg0) {
					connected.setRemove(removeExtra.getValue());
				}
			});
			rightPanel.add(removeExtra);
		}

		final CheckBox connect = new CheckBox("Connect");
		connect.setValue(!suggestedByKb); // automatically check/uncheck the value it based on the suggestion
		connect.setTitle("Connect this record");
		connect.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent arg0) {
				connected.setConnect(connect.getValue());
			}
		});
		rightPanel.add(connect);
		hp.add(leftPanel);
		hp.add(rightPanel);
		hp.getElement().getStyle().setPadding(10, Unit.PX);
		hp.getElement().getStyle().setMarginBottom(10, Unit.PX);
		return hp;
	}

	/**
	 * Builds up a widget for connecting records.
	 * @param w the widget
	 * @param cb the connectBean.
	 * @param acceptedDomain 
	 */
	private Tuple buildWidgetConnect(final ConnectedBean cb, final String acceptedDomain){

		VerticalPanel main = new VerticalPanel();
		main.setWidth("100%");
		HorizontalPanel hp = new HorizontalPanel();
		hp.setWidth("100%");

		VerticalPanel vpLeft = new VerticalPanel();
		vpLeft.getElement().getStyle().setMarginLeft(15, Unit.PX);
		vpLeft.setWidth("80%");
		Paragraph semanticIdentifier = new Paragraph("UUID:");
		final TextBox box = new TextBox();
		final Icon icon = new Icon(IconType.OK_SIGN);
		icon.setVisible(false);
		icon.getElement().getStyle().setMarginLeft(10, Unit.PX);
		icon.getElement().getStyle().setMarginTop(5, Unit.PX);
		final Anchor view = new Anchor();
		view.setText("View"); 
		view.setTitle("Click to inspect the record");
		view.setTarget("_blank");
		view.getElement().getStyle().setFontWeight(FontWeight.BOLD);
		view.setVisible(false);

		// add a couple of handlers
		box.addKeyPressHandler(new KeyPressHandler() {

			@Override
			public void onKeyPress(KeyPressEvent event) {
				GWT.log("onKeyPress " + event.getNativeEvent().getKeyCode());
				Scheduler.get().scheduleDeferred(new ScheduledCommand() {

					@Override
					public void execute() {
						box.setFocus(false);
					}
				});
			}
		});
		box.addChangeHandler(new ChangeHandler() {

			@Override
			public void onChange(ChangeEvent event) {
				GWT.log("onChange");
				validateUUID(box, cb, icon, view, acceptedDomain);
			}
		});
		box.setWidth("512px");
		box.setPlaceholder("Copy and Paste the Identifier (UUID) of the record to connect here");

		vpLeft.add(semanticIdentifier);
		vpLeft.add(box);

		VerticalPanel vpRight = new VerticalPanel();
		vpRight.setWidth("20%");

		Button removeExtra = new Button();
		removeExtra.setText("Remove");
		removeExtra.setType(ButtonType.LINK);
		removeExtra.getElement().getStyle().setFontWeight(FontWeight.BOLD);
		removeExtra.setTitle("Remove this Connection");
		removeExtra.addClickHandler(new ClickHandler() {

			@Override
			public void onClick(ClickEvent arg0) {

				// remove this object from the pairs list
				Iterator<Tuple> iterator = connectList.iterator();
				while (iterator.hasNext()) {
					Tuple pair = (Tuple) iterator
							.next();
					if(pair.getO().equals(cb)){
						pair.getW().removeFromParent();
						iterator.remove();
					}
				}
			}
		});
		vpRight.add(removeExtra);
		hp.add(vpLeft);
		hp.add(vpRight);
		HTML separator = new HTML("<hr style=\"width:100%;\"/>");
		connectPanel.add(separator);
		main.add(hp);
		main.add(separator);
		return new Tuple(cb, main, box);
	}

	protected void validateUUID(final TextBox box, final ConnectedBean c, final Icon icon, final Anchor view, final String acceptedDomain) {

		if(!box.isEnabled())
			return;

		final String currentText = box.getText().trim();
		c.setKnowledgeBaseId(null);
		c.setConnect(false);
		icon.setVisible(false);
		view.setVisible(false);
		icon.setSpin(false);

		if(currentText == null || currentText.isEmpty())
			return;

		if(!currentText.matches(REGEX_UUID)){
			icon.setType(IconType.BAN_CIRCLE);
			icon.setTitle("Not a valid UUID");
			icon.setVisible(true);
			view.setVisible(false);
			return;
		}

		// else check at server side if it exists
		GWT.log("Text changed to " + currentText);
		box.setEnabled(false);
		icon.setIcon(IconType.ROTATE_RIGHT);
		icon.setSpin(true);

		service.checkIdentifierExistsInDomain(currentText, acceptedDomain, new AsyncCallback<String>() {

			@Override
			public void onSuccess(String result) {

				icon.setSpin(false);
				if(result != null){
					c.setKnowledgeBaseId(currentText);
					c.setConnect(true);
					icon.setType(IconType.OK_CIRCLE);
					icon.setTitle("Accepted");
					view.setHref(result);
					view.setVisible(true);
					box.setEnabled(false);
				}
				else{
					icon.setType(IconType.BAN_CIRCLE);
					icon.setTitle("Not a valid UUID");
					view.setVisible(false);
					box.setEnabled(true);
				}
				icon.setVisible(true);
			}

			@Override
			public void onFailure(Throwable caught) {
				box.setEnabled(true);
				icon.setSpin(false);
				view.setVisible(false);
				icon.setType(IconType.BAN_CIRCLE);
				icon.setTitle(caught.getMessage());
			}
		});

	}

	/**
	 * Get the list of records to connect with this one
	 * @return
	 */
	public List<ConnectedBean> getConnectList() {

		List<ConnectedBean> toReturn = new ArrayList<>();

		if(currentlyConnected != null)
			toReturn.addAll(currentlyConnected);

		if(suggestedByKnowledgeBase != null)
			toReturn.addAll(suggestedByKnowledgeBase);

		for (Tuple p : connectList) {
			ConnectedBean potentialConnection = (ConnectedBean) p.getO();
			String suggestedIdentifier = potentialConnection.getKnowledgeBaseId();
			if(suggestedIdentifier == null || suggestedIdentifier.isEmpty())
				continue;
			else
				toReturn.add(potentialConnection);
		}

		return toReturn;

	}

}
