package org.gcube.datacatalogue.grsf_manage_widget.shared;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.gcube.datacatalogue.common.enums.Status;

/**
 * The bean to be managed by GRSF Editors and Reviewers.
 * @author Costantino Perciante at ISTI-CNR (costantino.perciante@isti.cnr.it)
 */
public class ManageProductBean implements Serializable{

	private static final long serialVersionUID = -4882608487467259326L;
	private String semanticIdentifier; // Stock id or Fishery id
	private String catalogueIdentifier; // Catalogue id
	private String knowledgeBaseIdentifier; // GRSF UUID
	private String grsfType; // Fishery or Stock type (e.g., Assessment_Unit, Marine Resource and so on)
	private String grsfDomain; // fishery/stock
	private String grsfName; // Fishery name or Stock name
	private String shortName; // it is editable ...
	private String shortNameUpdated; // the updated one, if any
	private boolean traceabilityFlag; //from false to true etc
	private Status currentStatus;
	private Status newStatus;
	private String annotation; // added by the administrator
	private List<SourceRecord> sources; // sources for this record
	private List<SimilarGRSFRecord> similarGrsfRecords;
	private List<ConnectedBean> suggestedByKnowledgeBaseConnections;
	private List<ConnectedBean> suggestdByAdministratorConnections = new ArrayList<ConnectedBean>(0);
	private List<ConnectedBean> currentConnections;
	private List<ConnectedBean> connections; // the one to used eventually
	private boolean mergesInvolved; // important: in this case an email must be sent to the editors/reviewers
	private String recordUrl; // this record url
	private String report; // the report that keeps track of the changes

	public ManageProductBean() {
		super();
	}

	public ManageProductBean(
			String semanticIdentifier,
			String catalogueIdentifier, 
			String knowledgeBaseIdentifier,
			String grsfType, 
			String grsfDomain, 
			String grsfName,
			String shortName, 
			boolean traceabilityFlag, 
			Status currentStatus,
			String recordUrl, 
			List<SourceRecord> sources,
			List<SimilarGRSFRecord> similarGrsfRecords, 
			List<ConnectedBean> currentConnections, 
			List<ConnectedBean> suggestedByKnowledgeBaseConnections
			) {
		super();
		this.semanticIdentifier = semanticIdentifier;
		this.catalogueIdentifier = catalogueIdentifier;
		this.knowledgeBaseIdentifier = knowledgeBaseIdentifier;
		this.grsfType = grsfType;
		this.grsfDomain = grsfDomain;
		this.grsfName = grsfName;
		this.shortName = shortName;
		this.shortNameUpdated = shortName;
		this.traceabilityFlag = traceabilityFlag;
		this.currentStatus = currentStatus;
		this.sources = sources;
		this.similarGrsfRecords = similarGrsfRecords;
		this.currentConnections = currentConnections;
		this.suggestedByKnowledgeBaseConnections = suggestedByKnowledgeBaseConnections;
		this.recordUrl = recordUrl;
	}

	public String getSemanticIdentifier() {
		return semanticIdentifier;
	}

	public void setSemanticIdentifier(String semanticIdentifier) {
		this.semanticIdentifier = semanticIdentifier;
	}

	public List<SourceRecord> getSources() {
		return sources;
	}

	public void setSources(List<SourceRecord> sources) {
		this.sources = sources;
	}

	public List<SimilarGRSFRecord> getSimilarGrsfRecords() {
		return similarGrsfRecords;
	}

	public void setSimilarGrsfRecords(List<SimilarGRSFRecord> similarGrsfRecords) {
		this.similarGrsfRecords = similarGrsfRecords;
	}

	public String getCatalogueIdentifier() {
		return catalogueIdentifier;
	}

	public void setCatalogueIdentifier(String catalogueIdentifier) {
		this.catalogueIdentifier = catalogueIdentifier;
	}

	public String getKnowledgeBaseIdentifier() {
		return knowledgeBaseIdentifier;
	}

	public void setKnowledgeBaseIdentifier(String knowledgeBaseIdentifier) {
		this.knowledgeBaseIdentifier = knowledgeBaseIdentifier;
	}

	public String getGrsfType() {
		return grsfType;
	}

	public void setGrsfType(String grsfType) {
		this.grsfType = grsfType;
	}

	public String getGrsfDomain() {
		return grsfDomain;
	}

	public void setGrsfDomain(String grsfDomain) {
		this.grsfDomain = grsfDomain;
	}

	public String getGrsfName() {
		return grsfName;
	}

	public void setGrsfName(String grsfName) {
		this.grsfName = grsfName;
	}

	public Status getCurrentStatus() {
		return currentStatus;
	}

	public void setCurrentStatus(Status currentStatus) {
		this.currentStatus = currentStatus;
	}

	public Status getNewStatus() {
		return newStatus;
	}

	public void setNewStatus(Status newStatus) {
		this.newStatus = newStatus;
	}

	public String getAnnotation() {
		return annotation;
	}

	public void setAnnotation(String annotation) {
		this.annotation = annotation;
	}

	public boolean isTraceabilityFlag() {
		return traceabilityFlag;
	}

	public void setTraceabilityFlag(boolean traceabilityFlag) {
		this.traceabilityFlag = traceabilityFlag;
	}

	public String getShortName() {
		return shortName;
	}

	public void setShortName(String shortName) {
		this.shortName = shortName;
	}

	public String getShortNameUpdated() {
		return shortNameUpdated;
	}

	public void setShortNameUpdated(String shortNameUpdated) {
		this.shortNameUpdated = shortNameUpdated;
	}

	public List<ConnectedBean> getSuggestedByKnowledgeBaseConnections() {
		return suggestedByKnowledgeBaseConnections;
	}

	public void setSuggestedByKnowledgeBaseConnections(
			List<ConnectedBean> suggestedByKnowledgeBaseConnections) {
		this.suggestedByKnowledgeBaseConnections = suggestedByKnowledgeBaseConnections;
	}

	public List<ConnectedBean> getSuggestdByAdministratorConnections() {
		return suggestdByAdministratorConnections;
	}

	public void setSuggestdByAdministratorConnections(
			List<ConnectedBean> suggestdByAdministratorConnections) {
		this.suggestdByAdministratorConnections = suggestdByAdministratorConnections;
	}

	public List<ConnectedBean> getCurrentConnections() {
		return currentConnections;
	}

	public void setCurrentConnections(List<ConnectedBean> currentConnections) {
		this.currentConnections = currentConnections;
	}

	public boolean isMergesInvolved() {
		return mergesInvolved;
	}

	public void setMergesInvolved(boolean mergesInvolved) {
		this.mergesInvolved = mergesInvolved;
	}

	public String getRecordUrl() {
		return recordUrl;
	}

	public void setRecordUrl(String recordUrl) {
		this.recordUrl = recordUrl;
	}

	public List<ConnectedBean> getConnections() {
		return connections;
	}

	public void setConnections(List<ConnectedBean> connections) {
		this.connections = connections;
	}

	public String getReport() {
		return report;
	}

	public void setReport(String report) {
		this.report = report;
	}

	@Override
	public String toString() {
		return "ManageProductBean [semanticIdentifier=" + semanticIdentifier
				+ ", catalogueIdentifier=" + catalogueIdentifier
				+ ", knowledgeBaseIdentifier=" + knowledgeBaseIdentifier
				+ ", grsfType=" + grsfType + ", grsfDomain=" + grsfDomain
				+ ", grsfName=" + grsfName + ", shortName=" + shortName
				+ ", shortNameUpdated=" + shortNameUpdated
				+ ", traceabilityFlag=" + traceabilityFlag + ", currentStatus="
				+ currentStatus + ", newStatus=" + newStatus + ", annotation="
				+ annotation + ", sources=" + sources + ", similarGrsfRecords="
				+ similarGrsfRecords + ", suggestedByKnowledgeBaseConnections="
				+ suggestedByKnowledgeBaseConnections + ", suggestdByAdministratorConnections="
				+ suggestdByAdministratorConnections + ", currentConnections="
				+ currentConnections + ", connections=" + connections
				+ ", mergesInvolved=" + mergesInvolved + ", recordUrl="
				+ recordUrl + ", report=" + report + "]";
	}

}
